//  Copyright (C) 1999 AT&T Laboratories Cambridge. All Rights Reserved.
//
//  This file is part of the VNC system.
//
//  The VNC system is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
// TightVNC distribution homepage on the Web: http://www.tightvnc.com/
//
// If the source code for the VNC system is not available from the place 
// whence you received this file, check http://www.uk.research.att.com/vnc or contact
// the authors on vnc@uk.research.att.com for information on obtaining it.


#include "stdhdrs.h"
#include "vncviewer.h"
#include "Exception.h"
#ifdef UNDER_CE
#include "omnithreadce.h"
#else
#include "omnithread.h"
#include "VNCviewerApp32.h"
#endif

//Pointdev Begin (http://www.pointdev.com/free/)
inline bool SwitchMatch(LPCTSTR arg, LPCTSTR swtch) {
	return (arg[0] == '-' || arg[0] == '/') &&
		(_tcsicmp(&arg[1], swtch) == 0);
}

BOOL ControlServ(char * ordis, char * servname, DWORD DControl, DWORD StatusRet)
{
	SC_HANDLE handle;
	SERVICE_STATUS pbuf;
	BOOL ok=FALSE;
	
	DWORD TypeService=SERVICE_WIN32;

	char bb[MAX_PATH];
	BOOL oksuppr=TRUE;

	for (size_t c=0; c<strlen(ordis); c++)
		if (ordis[c]==':')
			ordis[c]=0x0;

	wsprintf(bb,"\\\\%s",ordis);

	handle=OpenSCManager(bb,NULL,SC_MANAGER_ALL_ACCESS);

	if (handle!=NULL)
	{
		SC_HANDLE sh = OpenService(handle,servname,SERVICE_ALL_ACCESS);

		if (sh!=NULL)
		{
			if (oksuppr)
			{
				if (ControlService(sh, DControl, &pbuf)) 
				{
					DWORD err=pbuf.dwWin32ExitCode;
					DWORD timel=GetTickCount();

					Sleep(1000);

					pbuf.dwCurrentState=0;

					while (err==NO_ERROR && pbuf.dwCurrentState!=StatusRet && (GetTickCount()-timel<40000))
					{
						Sleep(100);
						QueryServiceStatus(sh,&pbuf);
						err=pbuf.dwWin32ExitCode;
						if (err!=NO_ERROR)
							err=pbuf.dwServiceSpecificExitCode;
					} 

					ok=(pbuf.dwCurrentState==StatusRet);
				}
			}
			CloseServiceHandle(sh);
		} 
		CloseServiceHandle(handle);
	} 

	return ok;
}

//Pointdev End (http://www.pointdev.com/free/)

// All logging is done via the log object
Log vnclog;

#ifdef UNDER_CE
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR szCmdLine, int iCmdShow)
#else
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
#endif
{
	// The state of the application as a whole is contained in the one app object
	#ifdef _WIN32_WCE
		VNCviewerApp app(hInstance, szCmdLine);
	#else
		VNCviewerApp32 app(hInstance, szCmdLine);
	#endif

	// Start a new connection if specified on command line, 
	// or if not in listening mode
	
	if (app.m_options.m_connectionSpecified) {
		app.NewConnection(app.m_options.m_host, app.m_options.m_port);
	} else if (!app.m_options.m_listening) {
		// This one will also read from config file if specified
		app.NewConnection();
	}

	MSG msg;
	
	try {
		while ( GetMessage(&msg, NULL, 0,0) ) {
			DispatchMessage(&msg);
		} 
	} catch (WarningException &e) {
		e.Report();
	} catch (QuietException &e) {
		e.Report();
	}
	
	// Clean up winsock
	WSACleanup();

	vnclog.Print(3, _T("Exiting\n"));

	//Pointdev Begin (http://www.pointdev.com/free/)
	int cmdlinelen = _tcslen(szCmdLine);
	if (cmdlinelen != 0) 
	{
		
		TCHAR *cmd = new TCHAR[cmdlinelen + 1];
		_tcscpy(cmd, szCmdLine);
		
		// Count the number of spaces
		// This may be more than the number of arguments, but that doesn't matter.
		int nspaces = 0;
		TCHAR *p = cmd;
		TCHAR *pos = cmd;
		while ( ( pos = _tcschr(p, ' ') ) != NULL ) {
			nspaces ++;
			p = pos + 1;
		}
		
		// Create the array to hold pointers to each bit of string
		TCHAR **args = new LPTSTR[nspaces + 1];
		
		// replace spaces with nulls and
		// create an array of TCHAR*'s which points to start of each bit.
		pos = cmd;
		int i = 0;
		args[i] = cmd;
		bool inquote=false;
		for (pos = cmd; *pos != 0; pos++) {
			// Arguments are normally separated by spaces, unless there's quoting
			if ((*pos == ' ') && !inquote) {
				*pos = '\0';
				p = pos + 1;
				args[++i] = p;
			}
			if (*pos == '"') {  
				if (!inquote) {      // Are we starting a quoted argument?
					args[i] = ++pos; // It starts just after the quote
				} else {
					*pos = '\0';     // Finish a quoted argument?
				}
				inquote = !inquote;
			}
		}
		i++;

		bool hostGiven = false, portGiven = false;
		// take in order.
		for (int j = 0; j < i; j++) {
			if ( SwitchMatch(args[j], _T("stopservice"))) 
				ControlServ(args[0],"winvnc",SERVICE_CONTROL_STOP,SERVICE_STOPPED);
		}
	}
	//Pointdev End (http://www.pointdev.com/free/)


	return msg.wParam;
}


// Move the given window to the centre of the screen
// and bring it to the top.
void CentreWindow(HWND hwnd)
{
	RECT winrect, workrect;
	
	// Find how large the desktop work area is
	SystemParametersInfo(SPI_GETWORKAREA, 0, &workrect, 0);
	int workwidth = workrect.right -  workrect.left;
	int workheight = workrect.bottom - workrect.top;
	
	// And how big the window is
	GetWindowRect(hwnd, &winrect);
	int winwidth = winrect.right - winrect.left;
	int winheight = winrect.bottom - winrect.top;
	// Make sure it's not bigger than the work area
	winwidth = min(winwidth, workwidth);
	winheight = min(winheight, workheight);

	// Now centre it
	SetWindowPos(hwnd, 
		HWND_TOP,
		workrect.left + (workwidth-winwidth) / 2,
		workrect.top + (workheight-winheight) / 2,
		winwidth, winheight, 
		SWP_SHOWWINDOW);
	SetForegroundWindow(hwnd);
}

// Convert "host:display" into host and port
// Returns true if valid format, false if not.
// Takes initial string, addresses of results and size of host buffer in wchars.
// If the display info passed in is longer than the size of the host buffer, it
// is assumed to be invalid, so false is returned.
bool ParseDisplay(LPTSTR display, LPTSTR phost, int hostlen, int *pport) 
{
    int tmpport;
    if (hostlen < (int) _tcslen(display))
        return false;
    TCHAR *colonpos = _tcschr( display, L':' );
    if (colonpos == NULL) {
        tmpport = 0;
		_tcsncpy(phost, display, MAX_HOST_NAME_LEN);
	} else {
		_tcsncpy(phost, display, colonpos-display);
		phost[colonpos-display] = L'\0';
		if (_stscanf(colonpos+1, TEXT("%d"), &tmpport) != 1) 
			return false;
	}
    if (tmpport < 100)
        tmpport += RFB_PORT_OFFSET;
    *pport = tmpport;
    return true;
}


